/**
 * File name    : UIDiagSynchronizationNew.java
 * Project name : PRJMRSTORAGE
 *
 * Created on 24.03.2009 11:50:47 by Didiot
 *
 * (c) 2009 Treofan GmbH&Co.KG Neunkirchen
 */
package com.sfc.prjmrstorage2.ui;

import java.beans.*;
import java.util.*;
import java.util.List;

import org.eclipse.jface.dialogs.*;
import org.eclipse.jface.viewers.*;
import org.eclipse.swt.*;
import org.eclipse.swt.events.*;
import org.eclipse.swt.layout.*;
import org.eclipse.swt.widgets.*;
import org.eclipse.swt.widgets.Dialog;

import com.sfc.libcore.SAPTools.*;
import com.sfc.libdac.extrusion.*;
import com.sfc.libui.*;
import com.sfc.libui.dialog.*;
import com.sfc.prjmrstorage2.*;
import com.sfc.prjmrstorage2.actions.*;
import com.sfc.prjmrstorage2.data.*;
import com.swtdesigner.*;

public final class UIDialogReconciliation extends Dialog {

	protected Object			result;
	protected Shell				shell;
	private Group				compositeRow;
	private Button				btnDoReconciliate;
	private Button				btnDoRead;
	private Composite			composite;
	private Table				table;
	private TableViewer			viewer;

	private boolean				isReadAutomatic;
	private Composite			composite_1;
	private Button				btnEditStorage;
	private ActionEditMRStorage	actionEditMRStorage;
    private Button              btnCheckAllRows;
	private List<Button>        listOfChekButtons;
    
	private Listener checkBtnListener = new Listener() {
		public void handleEvent(Event event) {
			for (Button button : listOfChekButtons) 
				if (button.getSelection()) {
					btnDoRead.setEnabled(true);
					return;
				}
			btnDoRead.setEnabled(false);
		};
	};
	
	
    /**
     * Create contents of the dialog
     * @param parent
     */
    protected Control createDialogArea(Composite parent) {
        this.listOfChekButtons = new ArrayList<Button>();
    	
    	parent.setLayout(new FillLayout());
        Composite container = new Composite(parent, SWT.NONE);
        
        final GridLayout gridLayout_1 = new GridLayout();
        gridLayout_1.numColumns = 2;
        container.setLayout(gridLayout_1);

        this.composite = new Composite(container, SWT.NONE);
        this.composite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false, 3, 1));
        final GridLayout gridLayout = new GridLayout();
        gridLayout.numColumns = 2;
        this.composite.setLayout(gridLayout);

        this.compositeRow = new Group(this.composite, SWT.NONE);
        this.compositeRow.setLayout(new RowLayout());
        this.compositeRow.setText("Reihen");
        this.compositeRow.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 2, 1));

        this.actionEditMRStorage = new ActionEditMRStorage();
        
        
//        if (Beans.isDesignTime()) 
//        	compositeRow.setText("Design time");
//        else 
//        	for (Integer rowNr : Model.instance().getListOfRowNumbers()) {
//        		Button button = new Button(compositeRow, SWT.CHECK);
//        		button.setText(String.valueOf(rowNr));
//        		button.setSelection(isReadAutomatic);
//        		button.addListener(SWT.Selection, checkBtnListener);
//        		listOfChekButtons.add(button);
//			}

        this.viewer = new TableViewer(container, SWT.BORDER | SWT.FULL_SELECTION | SWT.HIDE_SELECTION | SWT.MULTI);
        this.viewer.setUseHashlookup(true);
        this.viewer.addSelectionChangedListener(new ISelectionChangedListener() {
			@Override
			public void selectionChanged(SelectionChangedEvent event) {
		        StructuredSelection structuredSelection = (StructuredSelection)event.getSelection();
		        ReconciliationElement reconcElement = (ReconciliationElement)structuredSelection.getFirstElement();
		        actionEditMRStorage.setSelectedBinElement(reconcElement.getJUICeStorage());
			}
		});
        this.table = this.viewer.getTable();
        this.table.setLinesVisible(true);
        this.table.setHeaderVisible(true);
        this.table.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 3, 1));
        this.viewer.setContentProvider(new ArrayContentProvider());
        this.viewer.setLabelProvider(new MyCellLabelProvider());

        for (ETableColumns eColumns : ETableColumns.values()) 
            ETableColumns.createColumn(this.viewer, eColumns);
                                
        composite_1 = new Composite(container, SWT.NONE);
        composite_1.setLayout(new GridLayout(4, false));
        composite_1.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 3, 1));

        this.btnCheckAllRows = new Button(composite_1, SWT.CHECK);
        this.btnCheckAllRows.setText("Alle Reihen auswhlen");
        this.btnCheckAllRows.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 1));
        this.btnCheckAllRows.addListener(SWT.Selection, new Listener() {
			@Override
			public void handleEvent(Event event) {
				selectAllRows(btnCheckAllRows.getSelection());
			}
		});
        
        this.btnDoReconciliate = new Button(composite_1, SWT.NONE);
        btnDoReconciliate.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, true, false, 1, 1));
        this.btnDoReconciliate.setImage(SWTResourceManager.getImage(Settings.class, "/icons/save.gif"));
        this.btnDoReconciliate.addSelectionListener(new SelectionAdapter() {
        	public void widgetSelected(final SelectionEvent e) {
        		List<ReconciliationElement> reconElements = (List<ReconciliationElement>)viewer.getInput();
        		//Model.instance().doApplyReconciliationElements(reconElements);
        		btnDoReconciliate.setEnabled(false);
        		doReadReconciliationElements();
        	}
        });
        this.btnDoReconciliate.setText("Abgleichen");
        this.btnDoReconciliate.setEnabled(false);

        this.btnDoRead = new Button(composite_1, SWT.NONE);
        this.btnDoRead.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 1));
        this.btnDoRead.setToolTipText("Aus der SPS Daten einlesen, und Unterschiede anzeigen");
        this.btnDoRead.setImage(SWTResourceManager.getImage(Settings.class, "/icons/refresh.gif"));
        this.btnDoRead.setText("Abweichungen ermitteln");
        this.btnDoRead.setEnabled(false);
        this.btnDoRead.addSelectionListener(new SelectionAdapter() {
        	public void widgetSelected(final SelectionEvent e) {
        		doReadReconciliationElements();
        	}
        });
        
        return container;
    }

    /**
     * Create the dialog
     * @param parent
     * @param style
     */
    public UIDialogReconciliation(Shell parent, int style) {
        super(parent, style);
    }

    /**
     * Create the dialog
     * @param parent
     */
    public UIDialogReconciliation(Shell parent) {
        this(parent, SWT.NONE);
        
    }

    /**
     * Open the dialog
     * @return the result
     */
    public Object open() {
        createContents();
        this.shell.layout();
        UITools.centerOnScreen(shell);
        this.shell.open();
        
        if (isReadAutomatic) {
        	selectAllRows(true);
            this.doReadReconciliationElements();
        }
        
        Display display = getParent().getDisplay();
        while (!this.shell.isDisposed()) {
            if (!display.readAndDispatch())
                display.sleep();
        }
        return this.result;
    }

    /**
     * Create contents of the dialog
     */
    protected void createContents() {
        this.shell = new Shell(getParent(), SWT.TITLE | SWT.BORDER | SWT.APPLICATION_MODAL | SWT.CLOSE);
        this.shell.setSize(681, 438);
        if (!Beans.isDesignTime())
            shell.setText("Abgleich Lager : " + Settings.STOCK_NAME + " ffnung : " + Settings.HOLE_NUMBER);
        else
            shell.setText("Abgleich Lager : Design");
        shell.setImage(SWTResourceManager.getImage(UIDialogReconciliation.class, "/icons/reconcile.gif"));        
        
        createDialogArea(shell);
        
        shell.addListener(SWT.Close, new Listener() {
        	@Override
        	public void handleEvent(Event event) {
        		if (viewer.getTable().getItemCount() != 0) {
        			MessageDialog.openError(event.display.getActiveShell(),"JUICe/SPS Abweichungen liegen noch vor" , "Bitte erst Abweichungen verarbeiten!");
        			event.doit = false; 
        		}
        	}
        });
    }
    
    
    private void doReadReconciliationElements() {
    	viewer.getTable().removeAll();
    	String[] selectedRows = getSelectedRows();
    	try {
    		//List<ReconciliationElement> reconElements = Model.instance().requestReconciliationElements(selectedRows);
    		List<ReconciliationElement> reconElements = Collections.EMPTY_LIST;
    		
    		if (reconElements.size() == 0) {
    			String title = String.format("markiete Reihen : %s", Arrays.toString(selectedRows));
    			MessageDialog.openInformation(shell, title, "Es wurden keine Abweichungen gefunden.");
    			return;
    		} 
    		
            viewer.setInput(reconElements);
            btnDoReconciliate.setEnabled(viewer.getTable().getItemCount() > 0);
    		
        } catch (Throwable ex) {
            new UIDialogException(shell).open("Synchronization", ex);
        }
    }

    
    public String[] getSelectedRows() {
        ArrayList<String> al = new ArrayList<String>();
        for (Control control : compositeRow.getChildren()) 
            if (((Button)control).getSelection())
                al.add(((Button)control).getText());
        return al.toArray(new String[al.size()]);
    }

	public void setReadAutomatic(boolean isPLCPreRetrieveEnabled) {
		this.isReadAutomatic = isPLCPreRetrieveEnabled;
	}
	
	public void selectAllRows(boolean value) {
		for (Button btn	: this.listOfChekButtons) 
			btn.setSelection(value);
		this.btnDoRead.setEnabled(value);
	}
	
	
	
}

class MyCellLabelProvider extends CellLabelProvider {
    @Override
    public void update(ViewerCell cell) {
    	Table table = (Table)cell.getControl();
        ReconciliationElement reconcElement = (ReconciliationElement)cell.getElement();
        switch (ETableColumns.values()[cell.getColumnIndex()]) {
            case BIN:
                cell.setText(reconcElement.getJUICeStorage().sapBinString());
                cell.setBackground(SWTResourceManager.getColor(SWT.COLOR_INFO_BACKGROUND));
                break;
                
            case JCE_MARKED:
            	cell.setImage(reconcElement.isJUICeChecked() ? Settings.IMAGE_CHECKED : Settings.IMAGE_UNCHECKED);
                return;
                
            case PLC_MARKED:
            	cell.setImage(!reconcElement.isJUICeChecked() ? Settings.IMAGE_CHECKED : Settings.IMAGE_UNCHECKED);
            	cell.setBackground(SWTResourceManager.getColor(SWT.COLOR_INFO_BACKGROUND));
                return;
                
            case JCE_BATCH_NUMBER:
                cell.setText(reconcElement.getJUICeStorage().getBatchNumber());
                cell.setBackground(SWTResourceManager.getColor(SWT.COLOR_INFO_BACKGROUND));
                break;
                
            case JCE_MILLROLL_NUMBER:
            	// See bei column creation.
                break;
            
            case JCE_RESERVED:
                cell.setText(reconcElement.getJUICeStorage().isReserved() ? SAPObject.SAP_TRUE_VALUE : "");
                cell.setBackground(SWTResourceManager.getColor(SWT.COLOR_INFO_BACKGROUND));
                break;
                
            case JCE_LOCKED:
                cell.setText(reconcElement.getJUICeStorage().isLocked() ? SAPObject.SAP_TRUE_VALUE : "");
                cell.setBackground(SWTResourceManager.getColor(SWT.COLOR_INFO_BACKGROUND));
                break;
            
            case JCE_PERMANENTLY_LOCKED:
                cell.setText(reconcElement.getJUICeStorage().isPermanentlyLocked() ? SAPObject.SAP_TRUE_VALUE : "");
                cell.setBackground(SWTResourceManager.getColor(SWT.COLOR_INFO_BACKGROUND));
                break;
                
            case JCE_TRANSLATED_PLC_CODE:
                cell.setText(String.valueOf(reconcElement.getJUICeStorage().getTranslatedPLCCode()));
                cell.setBackground(SWTResourceManager.getColor(SWT.COLOR_INFO_BACKGROUND));
                break;
                
            case PLC_BATCH_NUMBER:
            	cell.setText(String.valueOf(reconcElement.getPLCStorage().getBatchNumber()));
            	cell.setBackground(SWTResourceManager.getColor(SWT.COLOR_INFO_BACKGROUND));
            	break;
                
            case PLC_RESERVED:
                cell.setText(reconcElement.getPLCStorage().isReserved() ? SAPObject.SAP_TRUE_VALUE : "");
                cell.setBackground(SWTResourceManager.getColor(SWT.COLOR_INFO_BACKGROUND));
                break;
                
            case PLC_LOCKED:
                cell.setText(reconcElement.getPLCStorage().isLocked() ? SAPObject.SAP_TRUE_VALUE : "");
                cell.setBackground(SWTResourceManager.getColor(SWT.COLOR_INFO_BACKGROUND));
                break;
            
            case PLC_PERMANENTLY_LOCKED:
                cell.setText(reconcElement.getPLCStorage().isPermanentlyLocked() ? SAPObject.SAP_TRUE_VALUE : "");
                cell.setBackground(SWTResourceManager.getColor(SWT.COLOR_INFO_BACKGROUND));
                break;
                
            case PLC_CODE:
                cell.setText(String.valueOf(reconcElement.getPLCStorage().getTranslatedPLCCode()));
                cell.setBackground(SWTResourceManager.getColor(SWT.COLOR_INFO_BACKGROUND));
                break;
                
            case SEPARATOR:
            	cell.setBackground(Display.getCurrent().getSystemColor(SWT.COLOR_GRAY));
            	break;
            	
            default:
                break;
        }
    }
}



enum ETableColumns {
	
    DUMMY                   ("",            SWT.LEFT,       "", ""),
    BIN                     ("PLATZ",       SWT.LEFT,       "PLATZ",        "MR-Lager Platz [Reihe-Platz]"),
    
    JCE_MARKED            	("JCe", 	    SWT.CENTER,     "JUICe",        ""),
    JCE_BATCH_NUMBER		("SAP-Nr.",     SWT.CENTER,     "1123456789",	"Chargennummer in JUICe"),
    JCE_MILLROLL_NUMBER 	("MR-Nr.  ",	SWT.RIGHT,     "1123456789",   "Maschinenrollnummer in JUICe"),
    JCE_RESERVED           	("R",           SWT.CENTER,     "X",            "Platz ist reserviert in JUICe"),
    JCE_LOCKED             	("G",           SWT.CENTER,     "X",            "Platz ist gesperrt in JUICe"),
    JCE_PERMANENTLY_LOCKED	("F",           SWT.CENTER,     "X",            "Platz ist permanent gesperrt in JUICe"),
    JCE_TRANSLATED_PLC_CODE ("ST",          SWT.RIGHT,      "XX",           "JUICe Platz-Status"),
    SEPARATOR				(" ",			SWT.RIGHT,		"X",			""),	
    PLC_MARKED              ("SPS",         SWT.CENTER,     "SPS",          ""),
    PLC_BATCH_NUMBER		("SAP-Nr.",     SWT.CENTER,     "1123456789",   "" ),
    PLC_RESERVED            ("R",         	SWT.CENTER,     "X",            "Platz ist reserviert in der SPS"),
    PLC_LOCKED              ("G",         	SWT.CENTER,     "X",            "Platz ist gesperrt in der SPS"),
    PLC_PERMANENTLY_LOCKED	("F",         	SWT.CENTER,     "X",            "Platz ist permanent gesperrt in der SPS"),
    PLC_CODE                ("ST",          SWT.RIGHT,      "XX",           "SPS Platz-Status");
    
    private String colName;
    private int    alignement; 
    private String pattern;
    private String toolTip;
    
    private ETableColumns(String colName, int aligment, String pattern, String toolTip) {
        this.colName    = colName;
        this.alignement = aligment;
        this.pattern    = pattern;
        this.toolTip    = toolTip;
    }
    
    public static void createColumn(final TableViewer tv, ETableColumns eColumn) {
        
        TableColumn tc = new TableColumn(tv.getTable(), eColumn.alignement);
        tc.setText(eColumn.pattern);
        tc.pack();
        tc.setText(eColumn.colName);
        tc.setToolTipText(eColumn.toolTip);
        
        tc.setResizable(false);
        
        if (eColumn == ETableColumns.JCE_MILLROLL_NUMBER) {
            TableViewerColumn tableViewerColumn = new TableViewerColumn(tv, tc);
            tableViewerColumn.setLabelProvider(new CellLabelProvider(){
                @Override
                public void update(ViewerCell cell) {
                    BinElement binElement = ((ReconciliationElement)cell.getElement()).getJUICeStorage();
                    String txt = binElement.getMillRollNumber() == 0 ? "" : String.valueOf(binElement.getMillRollNumber());
                    cell.setText(txt);
                    cell.setBackground(SWTResourceManager.getColor(SWT.COLOR_INFO_BACKGROUND));
                }
            }); 

            
            tableViewerColumn.setEditingSupport(new EditingSupport(tv){
                private CellEditor cellEditor;
                @Override
                protected void setValue(Object element, Object value) {
                    String       	mrNumber = value.toString();
                    MRStorage    	jceBin   = ((ReconciliationElement)element).getJUICeStorage();
                    
                    if (jceBin.isMillRoll()) {
                    	//Model.instance().dbReadJMillRoll(BatchMap.getBatchNumber(mrNumber));
                    	//jceBin.setMaterialName(Model.instance().getJMillRoll().getMaterialNumber());
                        //jceBin.setCurrentLength((int)Model.instance().getJMillRoll().getProducedWeight());
                    }
                    tv.update(element, null);
                }
            
                @Override
                protected Object getValue(Object element) {
                    BinElement binElement = ((ReconciliationElement)element).getJUICeStorage();
                    return binElement.getMillRollNumber();
                }
            
                @Override
                protected CellEditor getCellEditor(Object element) {
                    if (this.cellEditor == null)
                        this.cellEditor = new TextCellEditor(tv.getTable());
                    return this.cellEditor;
                }
            
                @Override
                protected boolean canEdit(Object element) {
                    return true;
                }
            });
            
        }
        
        
        if (eColumn == ETableColumns.JCE_MARKED) {
            TableViewerColumn tvc = new TableViewerColumn(tv, tc);
            tvc.setLabelProvider(new CellLabelProvider(){
            	@Override
            	public void update(ViewerCell cell) {
            		ReconciliationElement reconElement = (ReconciliationElement )cell.getElement();
            		cell.setImage(reconElement.isJUICeChecked() ? Settings.IMAGE_CHECKED : Settings.IMAGE_UNCHECKED);
            	}
            }); 
            
            tvc.setEditingSupport(new EditingSupport(tv) {
                CheckboxCellEditor checkboxCellEditor;

                @Override
                protected boolean canEdit(Object element) {
                    return true;
                }

                @Override
                protected CellEditor getCellEditor(Object element) {
                    if (checkboxCellEditor == null) 
                        checkboxCellEditor = new CheckboxCellEditor(tv.getTable(), SWT.CHECK);
                    checkboxCellEditor.setValue(((ReconciliationElement)element).isJUICeChecked());
                    return checkboxCellEditor;
                }

                @Override
                protected Object getValue(Object element) {
                    return ((ReconciliationElement)element).isJUICeChecked();
                }

                @Override
                protected void setValue(Object element, Object value) {
                    ReconciliationElement so = (ReconciliationElement)element;
                    so.setJUICeChecked((Boolean)value);
                    tv.update(element, null);
                }
            });
        }
        
        if (eColumn == ETableColumns.PLC_MARKED) {
            TableViewerColumn tvc = new TableViewerColumn(tv, tc);
            tvc.setLabelProvider(new CellLabelProvider(){
            	@Override
            	public void update(ViewerCell cell) {
            		ReconciliationElement reconElement = (ReconciliationElement )cell.getElement();
            		cell.setImage(reconElement.isJUICeChecked() ? Settings.IMAGE_UNCHECKED : Settings.IMAGE_CHECKED);
            	}
            }); 
            
            tvc.setEditingSupport(new EditingSupport(tv) {
                CheckboxCellEditor checkboxCellEditor;

                @Override
                protected boolean canEdit(Object element) {
                    return true;
                }

                @Override
                protected CellEditor getCellEditor(Object element) {
                    if (checkboxCellEditor == null)
                        checkboxCellEditor = new CheckboxCellEditor(tv.getTable(), SWT.CHECK);
                    checkboxCellEditor.setValue(((ReconciliationElement)element).isJUICeChecked());
                    return checkboxCellEditor;
                }

                @Override
                protected Object getValue(Object element) {
                    return ((ReconciliationElement)element).isJUICeChecked();
                }

                @Override
                protected void setValue(Object element, Object value) {
                    ReconciliationElement so = (ReconciliationElement)element;
                    so.setJUICeChecked((Boolean)value);
                    tv.update(element, null);
                }
            });
        }

    }
}



